@@ -16,7 +16,7 @@ function send_change(jsdoc, event) {
     pyodideWorker.queue = events
     return
   }
-  const patch = jsdoc.create_json_patch_string([event])
+  const patch = jsdoc.create_json_patch([event])
   pyodideWorker.busy = true
   pyodideWorker.postMessage({type: 'patch', patch: patch})
 }
@@ -53,11 +53,11 @@ pyodideWorker.onmessage = async (event) => {
     const root_ids = JSON.parse(msg.root_ids)
 
     // Remap roots in message to element IDs
-    const root_els = document.getElementsByClassName('bk-root')
+    const root_els = document.querySelectorAll('[data-root-id]')
     const data_roots = []
     for (const el of root_els) {
        el.innerHTML = ''
-       data_roots.push([parseInt(el.getAttribute('data-root-id')), el.id])
+       data_roots.push([el.getAttribute('data-root-id'), el.id])
     }
     data_roots.sort((a, b) => a[0]<b[0] ? -1: 1)
     const roots = {}
@@ -71,17 +71,17 @@ pyodideWorker.onmessage = async (event) => {
     const [views] = await Bokeh.embed.embed_items(docs_json, render_items)
 
     // Remove loading spinner and message
-    body.classList.remove("bk", "pn-loading", "{{ loading_spinner }}")
+    body.classList.remove("pn-loading", "{{ loading_spinner }}")
     for (const loading_msg of loading_msgs) {
       loading_msg.remove()
     }
 
     // Setup bi-directional syncing
-    pyodideWorker.jsdoc = jsdoc = views[0].model.document
+    pyodideWorker.jsdoc = jsdoc = [...views.roots.values()][0].model.document
     jsdoc.on_change(send_change.bind(null, jsdoc), false)
     pyodideWorker.postMessage({'type': 'rendered'})
     pyodideWorker.postMessage({'type': 'location', location: JSON.stringify(window.location)})
   } else if (msg.type === 'patch') {
-    pyodideWorker.jsdoc.apply_json_patch(JSON.parse(msg.patch), msg.buffers, setter_id='py')
+    pyodideWorker.jsdoc.apply_json_patch(msg.patch, msg.buffers, setter_id='py')
   }
 };
