@@ -7,9 +7,6 @@ import re
 import urllib.parse as urlparse
 import uuid
 
-from typing import List, Tuple, Type
-
-import pkg_resources
 import tornado
 
 from bokeh.server.auth_provider import AuthProvider
@@ -19,8 +16,9 @@ from tornado.httputil import url_concat
 from tornado.web import RequestHandler
 
 from .config import config
+from .entry_points import entry_points_for
 from .io import state
-from .io.resources import ERROR_TEMPLATE, _env
+from .io.resources import BASIC_LOGIN_TEMPLATE, ERROR_TEMPLATE, _env
 from .util import base64url_decode, base64url_encode
 
 log = logging.getLogger(__name__)
@@ -217,7 +215,7 @@ class OAuthLoginHandler(tornado.web.RequestHandler):
             return
 
         log.debug("%s received user information.", type(self).__name__)
-        return self._on_auth(user, body['access_token'])
+        return self._on_auth(user, body['access_token'], body.get('refresh_token'))
 
     def get_state_cookie(self):
         """Get OAuth state from cookies
@@ -323,7 +321,7 @@ class OAuthLoginHandler(tornado.web.RequestHandler):
             self.set_state_cookie(state)
             await self.get_authenticated_user(**params)
 
-    def _on_auth(self, user_info, access_token):
+    def _on_auth(self, user_info, access_token, refresh_token=None):
         user_key = config.oauth_jwt_user or self._USER_KEY
         user = user_info[user_key]
         self.set_secure_cookie('user', user, expires_days=config.oauth_expiry)
@@ -331,8 +329,12 @@ class OAuthLoginHandler(tornado.web.RequestHandler):
         if state.encryption:
             access_token = state.encryption.encrypt(access_token.encode('utf-8'))
             id_token = state.encryption.encrypt(id_token.encode('utf-8'))
+            if refresh_token:
+                refresh_token = state.encryption.encrypt(refresh_token.encode('utf-8'))
         self.set_secure_cookie('access_token', access_token, expires_days=config.oauth_expiry)
         self.set_secure_cookie('id_token', id_token, expires_days=config.oauth_expiry)
+        if refresh_token:
+            self.set_secure_cookie('refresh_token', refresh_token, expires_days=config.oauth_expiry)
         return user
 
     def _on_error(self, response, body=None):
@@ -349,7 +351,7 @@ class OAuthLoginHandler(tornado.web.RequestHandler):
             log.warning(f"{provider} OAuth provider failed to fully "
                         f"authenticate returning the following response:"
                         f"{body}.")
-        raise HTTPError(500, f"{provider} authentication failed")
+        raise HTTPError(500, f"{provider} authentication failed {body}")
 
 
 class GenericLoginHandler(OAuthLoginHandler, OAuth2Mixin):
@@ -628,9 +630,9 @@ class OAuthIDTokenLoginHandler(OAuthLoginHandler):
 
         access_token = body['access_token']
         id_token = body['id_token']
-        return self._on_auth(id_token, access_token)
+        return self._on_auth(id_token, access_token, body.get('refresh_token'))
 
-    def _on_auth(self, id_token, access_token):
+    def _on_auth(self, id_token, access_token, refresh_token=None):
         decoded = decode_id_token(id_token)
         user_key = config.oauth_jwt_user or self._USER_KEY
         if user_key in decoded:
@@ -643,8 +645,12 @@ class OAuthIDTokenLoginHandler(OAuthLoginHandler):
         if state.encryption:
             access_token = state.encryption.encrypt(access_token.encode('utf-8'))
             id_token = state.encryption.encrypt(id_token.encode('utf-8'))
+            if refresh_token:
+                refresh_token = state.encryption.encrypt(refresh_token.encode('utf-8'))
         self.set_secure_cookie('access_token', access_token, expires_days=config.oauth_expiry)
         self.set_secure_cookie('id_token', id_token, expires_days=config.oauth_expiry)
+        if refresh_token:
+            self.set_secure_cookie('refresh_token', refresh_token, expires_days=config.oauth_expiry)
         return user
 
 
@@ -796,20 +802,6 @@ class OAuthProvider(AuthProvider):
             return request_handler.get_secure_cookie("user", max_age_days=config.oauth_expiry)
         return get_user
 
-    @property
-    def endpoints(self) -> List[Tuple[str, Type[RequestHandler]]]:
-        ''' URL patterns for login/logout endpoints.
-
-        '''
-        endpoints: List[Tuple[str, Type[RequestHandler]]] = []
-        if self.login_handler:
-            assert self.login_url is not None
-            endpoints.append(('/login', self.login_handler))
-        if self.logout_handler:
-            assert self.logout_url is not None
-            endpoints.append(('/logout', self.logout_handler))
-        return endpoints
-
     @property
     def login_url(self):
         if config.oauth_redirect_uri is None:
@@ -833,6 +825,68 @@ class OAuthProvider(AuthProvider):
         return LogoutHandler
 
 
+class BasicLoginHandler(RequestHandler):
+
+    def get(self):
+        try:
+            errormessage = self.get_argument("error")
+        except Exception:
+            errormessage = ""
+        self.write(self._basic_login_template.render(errormessage=errormessage))
+
+    def _validate(self, username, password):
+        if os.path.isfile(config.basic_auth):
+            with open(config.basic_auth, encoding='utf-8') as auth_file:
+                auth_info = json.loads(auth_file.read())
+            if username not in auth_info:
+                return False
+            return password == auth_info[username]
+        elif password == config.basic_auth:
+            return True
+        return False
+
+    def post(self):
+        username = self.get_argument("username", "")
+        password = self.get_argument("password", "")
+        auth = self._validate(username, password)
+        if auth:
+            self.set_current_user(username)
+            self.redirect("/")
+        else:
+            error_msg = "?error=" + tornado.escape.url_escape("Login incorrect")
+            self.redirect('/login' + error_msg)
+
+    def set_current_user(self, user):
+        if not user:
+            self.clear_cookie("user")
+            return
+        self.set_secure_cookie("user", user, expires_days=config.oauth_expiry)
+        id_token = base64url_encode(json.dumps({'user': user}))
+        if state.encryption:
+            id_token = state.encryption.encrypt(id_token.encode('utf-8'))
+        self.set_secure_cookie('id_token', id_token, expires_days=config.oauth_expiry)
+
+
+
+class BasicProvider(OAuthProvider):
+    def __init__(self, basic_login_template=None):
+        if basic_login_template is None:
+            self._basic_login_template = BASIC_LOGIN_TEMPLATE
+        else:
+            with open(basic_login_template) as f:
+                self._basic_login_template = _env.from_string(f.read())
+        super().__init__()
+
+    @property
+    def login_url(self):
+        return '/login'
+
+    @property
+    def login_handler(self):
+        BasicLoginHandler._basic_login_template = self._basic_login_template
+        return BasicLoginHandler
+
+
 AUTH_PROVIDERS = {
     'auth0': Auth0Handler,
     'azure': AzureAdLoginHandler,
@@ -846,7 +900,7 @@ AUTH_PROVIDERS = {
 }
 
 # Populate AUTH Providers from external extensions
-for entry_point in pkg_resources.iter_entry_points('panel.auth'):
-    AUTH_PROVIDERS[entry_point.name] = entry_point.resolve()
+for entry_point in entry_points_for('panel.auth'):
+    AUTH_PROVIDERS[entry_point.name] = entry_point.load()
 
 config.param.objects(False)['_oauth_provider'].objects = list(AUTH_PROVIDERS.keys())
