@@ -10,11 +10,12 @@ from typing import (
     IO, Any, Dict, List,
 )
 
+import bokeh
+
 from bokeh.application.application import Application, SessionContext
 from bokeh.application.handlers.code import CodeHandler
-from bokeh.command.util import build_single_handler_application
 from bokeh.core.json_encoder import serialize_json
-from bokeh.core.templates import FILE, MACROS, _env
+from bokeh.core.templates import MACROS, get_env
 from bokeh.document import Document
 from bokeh.embed.elements import script_for_render_items
 from bokeh.embed.util import RenderItem, standalone_docs_json_and_render_items
@@ -24,10 +25,11 @@ from typing_extensions import Literal
 
 from .. import __version__, config
 from ..util import base_version, escape
+from .markdown import build_single_handler_application
 from .mime_render import find_imports
 from .resources import (
-    CDN_DIST, DIST_DIR, INDEX_TEMPLATE, Resources, _env as _pn_env,
-    bundle_resources, set_resource_mode,
+    BASE_TEMPLATE, CDN_DIST, DIST_DIR, INDEX_TEMPLATE, Resources,
+    _env as _pn_env, bundle_resources, loading_css, set_resource_mode,
 )
 from .state import set_curdoc, state
 
@@ -37,17 +39,19 @@ WEB_WORKER_TEMPLATE = _pn_env.get_template('pyodide_worker.js')
 WORKER_HANDLER_TEMPLATE  = _pn_env.get_template('pyodide_handler.js')
 
 PANEL_ROOT = pathlib.Path(__file__).parent.parent
-BOKEH_VERSION = '2.4.3'
+BOKEH_VERSION = base_version(bokeh.__version__)
 PY_VERSION = base_version(__version__)
 PANEL_LOCAL_WHL = DIST_DIR / 'wheels' / f'panel-{__version__.replace("-dirty", "")}-py3-none-any.whl'
 BOKEH_LOCAL_WHL = DIST_DIR / 'wheels' / f'bokeh-{BOKEH_VERSION}-py3-none-any.whl'
 PANEL_CDN_WHL = f'{CDN_DIST}wheels/panel-{PY_VERSION}-py3-none-any.whl'
 BOKEH_CDN_WHL = f'{CDN_DIST}wheels/bokeh-{BOKEH_VERSION}-py3-none-any.whl'
-PYODIDE_URL = 'https://cdn.jsdelivr.net/pyodide/v0.22.1/full/pyodide.js'
+PYODIDE_URL = 'https://cdn.jsdelivr.net/pyodide/v0.23.0/full/pyodide.js'
 PYSCRIPT_CSS = '<link rel="stylesheet" href="https://pyscript.net/releases/2022.12.1/pyscript.css" />'
 PYSCRIPT_JS = '<script defer src="https://pyscript.net/releases/2022.12.1/pyscript.js"></script>'
 PYODIDE_JS = f'<script src="{PYODIDE_URL}"></script>'
 
+MINIMUM_VERSIONS = {}
+
 ICON_DIR = DIST_DIR / 'images'
 PWA_IMAGES = [
     ICON_DIR / 'favicon.ico',
@@ -172,6 +176,7 @@ def script_to_html(
     panel_version: Literal['auto', 'local'] | str = 'auto',
     manifest: str | None = None,
     http_patch: bool = True,
+    inline: bool = False
 ) -> str:
     """
     Converts a Panel or Bokeh script to a standalone WASM Python
@@ -180,30 +185,33 @@ def script_to_html(
     Arguments
     ---------
     filename: str | Path | IO
-      The filename of the Panel/Bokeh application to convert.
+        The filename of the Panel/Bokeh application to convert.
     requirements: 'auto' | List[str]
-      The list of requirements to include (in addition to Panel).
+        The list of requirements to include (in addition to Panel).
     js_resources: 'auto' | List[str]
-      The list of JS resources to include in the exported HTML.
+        The list of JS resources to include in the exported HTML.
     css_resources: 'auto' | List[str] | None
-      The list of CSS resources to include in the exported HTML.
+        The list of CSS resources to include in the exported HTML.
     runtime: 'pyodide' | 'pyscript'
-      The runtime to use for running Python in the browser.
+        The runtime to use for running Python in the browser.
     prerender: bool
-      Whether to pre-render the components so the page loads.
+        Whether to pre-render the components so the page loads.
     panel_version: 'auto' | str
-      The panel release version to use in the exported HTML.
+        The panel release version to use in the exported HTML.
     http_patch: bool
         Whether to patch the HTTP request stack with the pyodide-http library
         to allow urllib3 and requests to work.
+    inline: bool
+        Whether to inline resources.
     """
     # Run script
     if hasattr(filename, 'read'):
         handler = CodeHandler(source=filename.read(), filename='convert.py')
+        app_name = f'app-{str(uuid.uuid4())}'
         app = Application(handler)
     else:
         path = pathlib.Path(filename)
-        name = '.'.join(path.name.split('.')[:-1])
+        app_name = '.'.join(path.name.split('.')[:-1])
         app = build_single_handler_application(str(path.absolute()))
     document = Document()
     document._session_context = lambda: MockSessionContext(document=document)
@@ -222,13 +230,13 @@ def script_to_html(
     if requirements == 'auto':
         requirements = find_imports(source)
     elif isinstance(requirements, str) and pathlib.Path(requirements).is_file():
-        requirements = pathlib.Path(requirements).read_text().split('/n')
+        requirements = pathlib.Path(requirements).read_text(encoding='utf-8').split('/n')
         try:
-            import pkg_resources
-            parsed = pkg_resources.parse_requirements(requirements)
-            requirements = [str(requirement) for requirement in parsed]
-        except ImportError:
-            pass
+            from packaging.requirements import Requirement
+            requirements = [
+                r2 for r in requirements
+                if (r2 := r.split("#")[0].strip()) and Requirement(r2)
+            ]
         except Exception as e:
             raise ValueError(
                 f'Requirements parser raised following error: {e}'
@@ -246,10 +254,13 @@ def script_to_html(
         bokeh_req = f'bokeh=={BOKEH_VERSION}'
     base_reqs = [bokeh_req, panel_req]
     if http_patch:
-        base_reqs.append('pyodide-http==0.1.0')
+        base_reqs.append('pyodide-http==0.2.1')
     reqs = base_reqs + [
         req for req in requirements if req not in ('panel', 'bokeh')
     ]
+    for name, min_version in MINIMUM_VERSIONS.items():
+        if any(name in req for req in reqs):
+            reqs = [f'{name}>={min_version}' if name in req else req for req in reqs]
 
     # Execution
     post_code = POST_PYSCRIPT if runtime == 'pyscript' else POST
@@ -274,7 +285,7 @@ def script_to_html(
             if js_resources == 'auto':
                 js_resources = []
             worker_handler = WORKER_HANDLER_TEMPLATE.render({
-                'name': name,
+                'name': app_name,
                 'loading_spinner': config.loading_spinner
             })
             web_worker = WEB_WORKER_TEMPLATE.render({
@@ -308,11 +319,16 @@ def script_to_html(
         render_items = [render_item]
 
     # Collect resources
-    resources = Resources(mode='cdn')
+    resources = Resources(mode='inline' if inline else 'cdn')
+    loading_base = (DIST_DIR / "css" / "loading.css").read_text(encoding='utf-8')
+    spinner_css = loading_css()
+    css_resources.append(
+        f'<style type="text/css">\n{loading_base}\n{spinner_css}\n</style>'
+    )
     with set_curdoc(document):
         bokeh_js, bokeh_css = bundle_resources(document.roots, resources)
     extra_js = [INIT_SERVICE_WORKER, bokeh_js] if manifest else [bokeh_js]
-    bokeh_js = '\n'.join(extra_js+js_resources)
+    bokeh_js = '\n'.join(js_resources+extra_js)
     bokeh_css = '\n'.join([bokeh_css]+css_resources)
 
     # Configure template
@@ -320,26 +336,27 @@ def script_to_html(
     template_variables = document._template_variables
     context = template_variables.copy()
     context.update(dict(
-        title = document.title,
-        bokeh_js = bokeh_js,
-        bokeh_css = bokeh_css,
-        plot_script = plot_script,
-        docs = render_items,
-        base = FILE,
-        macros = MACROS,
-        doc = render_item,
-        roots = render_item.roots,
-        manifest = manifest
+        title=document.title,
+        bokeh_js=bokeh_js,
+        bokeh_css=bokeh_css,
+        plot_script=plot_script,
+        docs=render_items,
+        base=BASE_TEMPLATE,
+        macros=MACROS,
+        doc=render_item,
+        roots=render_item.roots,
+        manifest=manifest,
+        dist_url=CDN_DIST
     ))
 
     # Render
     if template is None:
-        template = FILE
+        template = BASE_TEMPLATE
     elif isinstance(template, str):
-        template = _env.from_string("{% extends base %}\n" + template)
+        template = get_env().from_string("{% extends base %}\n" + template)
     html = template.render(context)
     html = (html
-        .replace('<body>', f'<body class="bk pn-loading {config.loading_spinner}">')
+        .replace('<body>', f'<body class="pn-loading {config.loading_spinner}">')
     )
     return html, web_worker
 
@@ -353,6 +370,7 @@ def convert_app(
     manifest: str | None = None,
     panel_version: Literal['auto', 'local'] | str = 'auto',
     http_patch: bool = True,
+    inline: bool = False,
     verbose: bool = True,
 ):
     if dest_path is None:
@@ -361,11 +379,12 @@ def convert_app(
         dest_path = pathlib.Path(dest_path)
 
     try:
-        with set_resource_mode('cdn'):
+        with set_resource_mode('inline' if inline else 'cdn'):
             html, js_worker = script_to_html(
                 app, requirements=requirements, runtime=runtime,
                 prerender=prerender, manifest=manifest,
-                panel_version=panel_version, http_patch=http_patch
+                panel_version=panel_version, http_patch=http_patch,
+                inline=inline
             )
     except KeyboardInterrupt:
         return
@@ -389,6 +408,7 @@ def _convert_process_pool(
     apps: List[str],
     dest_path: str | None = None,
     max_workers: int = 4,
+    requirements: List[str] | Literal['auto'] | os.PathLike = 'auto',
     **kwargs
 ):
     import multiprocessing as mp
@@ -403,7 +423,13 @@ def _convert_process_pool(
         ) as executor:
             futures = []
             for app in group:
-                f = executor.submit(convert_app, app, dest_path, **kwargs)
+                if isinstance(requirements, dict):
+                    app_requires = requirements.get(app, 'auto')
+                else:
+                    app_requires = requirements
+                f = executor.submit(
+                    convert_app, app, dest_path, requirements=app_requires, **kwargs
+                )
                 futures.append(f)
             for future in concurrent.futures.as_completed(futures):
                 result = future.result()
@@ -425,6 +451,7 @@ def convert_apps(
     max_workers: int = 4,
     panel_version: Literal['auto', 'local'] | str = 'auto',
     http_patch: bool = True,
+    inline: bool = False,
     verbose: bool = True,
 ):
     """
@@ -439,7 +466,7 @@ def convert_apps(
         name for the application cache to ensure.
     runtime: 'pyodide' | 'pyscript' | 'pyodide-worker'
         The runtime to use for running Python in the browser.
-    requirements: 'auto' | List[str] | os.PathLike
+    requirements: 'auto' | List[str] | os.PathLike | Dict[str, 'auto' | List[str] | os.PathLike]
         The list of requirements to include (in addition to Panel).
         By default automatically infers dependencies from imports
         in the application. May also provide path to a requirements.txt
@@ -464,6 +491,8 @@ def convert_apps(
     http_patch: bool
         Whether to patch the HTTP request stack with the pyodide-http library
         to allow urllib3 and requests to work.
+    inline: bool
+        Whether to inline resources.
     """
     if isinstance(apps, str):
         apps = [apps]
@@ -475,12 +504,24 @@ def convert_apps(
 
     manifest = 'site.webmanifest' if build_pwa else None
 
+    if isinstance(requirements, dict):
+        app_requirements = {}
+        for app in apps:
+            matches = [
+                deps for name, deps in requirements.items()
+                if app.endswith(name.replace(os.path.sep, '/'))
+            ]
+            app_requirements[app] = matches[0] if matches else 'auto'
+    else:
+        app_requirements = requirements
+
     kwargs = {
-        'requirements': requirements, 'runtime': runtime,
+        'requirements': app_requirements, 'runtime': runtime,
         'prerender': prerender, 'manifest': manifest,
         'panel_version': panel_version, 'http_patch': http_patch,
-        'verbose': verbose
+        'inline': inline, 'verbose': verbose
     }
+
     if state._is_pyodide:
         files = dict((convert_app(app, dest_path, **kwargs) for app in apps))
     else:
