@@ -1,19 +1,31 @@
 """
 Layout components to lay out objects in a grid.
 """
+from __future__ import annotations
+
 import math
 
 from collections import OrderedDict, namedtuple
 from functools import partial
+from typing import (
+    TYPE_CHECKING, Any, ClassVar, Dict, List, Mapping, Optional, Tuple,
+)
 
 import numpy as np
 import param
 
-from bokeh.models import Box as BkBox, GridBox as BkGridBox
+from bokeh.models import FlexBox as BkFlexBox, GridBox as BkGridBox
 
 from ..io.model import hold
-from .base import _col, _row, ListPanel, Panel
+from ..io.resources import CDN_DIST
+from .base import (
+    ListPanel, Panel, _col, _row,
+)
 
+if TYPE_CHECKING:
+    from bokeh.document import Document
+    from bokeh.model import Model
+    from pyviz_comms import Comm
 
 
 class GridBox(ListPanel):
@@ -41,11 +53,21 @@ class GridBox(ListPanel):
     ncols = param.Integer(default=None, bounds=(0, None),  doc="""
       Number of columns to reflow the layout into.""")
 
-    _bokeh_model = BkGridBox
+    _bokeh_model: ClassVar[Model] = BkGridBox
 
-    _rename = {'objects': 'children', 'nrows': None, 'ncols': None}
+    _linked_properties: ClassVar[Tuple[str]] = ()
 
-    _source_transforms = {'scroll': None, 'objects': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'objects': 'children'
+    }
+
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'scroll': None, 'objects': None
+    }
+
+    _stylesheets: ClassVar[List[str]] = [
+        f'{CDN_DIST}css/gridbox.css'
+    ]
 
     @classmethod
     def _flatten_grid(cls, layout, nrows=None, ncols=None):
@@ -72,9 +94,9 @@ class GridBox(ListPanel):
                 if not children:
                     return Grid(0, 0, [])
 
-                nrows = lcm(*[ child.nrows for child in children ])
+                nrows = lcm(*[child.nrows for child in children])
                 if not ncols: # This differs from bokeh.layout.grid
-                    ncols = sum([ child.ncols for child in children ])
+                    ncols = sum([child.ncols for child in children])
 
                 items = []
                 offset = 0
@@ -139,19 +161,42 @@ class GridBox(ListPanel):
             layout = traverse(children)
         return cls._flatten_grid(layout, nrows, ncols)
 
+    def _compute_css_classes(self, children):
+        equal_widths, equal_heights = True, True
+        for (child, _, _, _, _) in children:
+            if child.sizing_mode and (child.sizing_mode.endswith('_both') or child.sizing_mode.endswith('_width')):
+                equal_widths &= True
+            else:
+                equal_widths = False
+            if child.sizing_mode and (child.sizing_mode.endswith('_both') or child.sizing_mode.endswith('_height')):
+                equal_heights &= True
+            else:
+                equal_heights = False
+        css_classes = []
+        if equal_widths:
+            css_classes.append('equal-width')
+        if equal_heights:
+            css_classes.append('equal-height')
+        return css_classes
+
     def _get_model(self, doc, root=None, parent=None, comm=None):
         model = self._bokeh_model()
-        if root is None:
-            root = model
-        objects = self._get_objects(model, [], doc, root, comm)
-        properties = self._process_param_change(self._init_params())
-        properties['children'] = self._get_children(objects, self.nrows, self.ncols)
-        model.update(**properties)
+        root = root or model
         self._models[root.ref['id']] = (model, parent)
-        self._link_props(model, self._linked_props, doc, root, comm)
+        objects = self._get_objects(model, [], doc, root, comm)
+        children = self._get_children(objects, self.nrows, self.ncols)
+        css_classes = self._compute_css_classes(children)
+        properties = {k: v for k, v in self._get_properties(doc).items() if k not in ('ncols', 'nrows')}
+        properties['css_classes'] = css_classes + properties.get('css_classes', [])
+        properties['children'] = children
+        model.update(**properties)
+        self._link_props(model, self._linked_properties, doc, root, comm)
         return model
 
-    def _update_model(self, events, msg, root, model, doc, comm=None):
+    def _update_model(
+        self, events: Dict[str, param.parameterized.Event], msg: Dict[str, Any],
+        root: Model, model: Model, doc: Document, comm: Optional[Comm]
+    ) -> None:
         from ..io import state
 
         msg = dict(msg)
@@ -194,17 +239,21 @@ class GridSpec(Panel):
     nrows = param.Integer(default=None, bounds=(0, None), doc="""
         Limits the number of rows that can be assigned.""")
 
-    width = param.Integer(default=600)
+    _bokeh_model: ClassVar[Model] = BkGridBox
 
-    height = param.Integer(default=600)
+    _linked_properties: ClassVar[Tuple[str]] = ()
 
-    _bokeh_model = BkGridBox
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'objects': 'children', 'mode': None, 'ncols': None, 'nrows': None
+    }
 
-    _source_transforms = {'objects': None, 'mode': None}
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'objects': None, 'mode': None
+    }
 
-    _rename = {'objects': 'children', 'mode': None, 'ncols': None, 'nrows': None}
+    _preprocess_params: ClassVar[List[str]] = ['objects']
 
-    _preprocess_params = ['objects']
+    _stylesheets: ClassVar[List[str]] = [f'{CDN_DIST}css/gridspec.css']
 
     def __init__(self, **params):
         if 'objects' not in params:
@@ -248,13 +297,13 @@ class GridSpec(Panel):
     def _get_objects(self, model, old_objects, doc, root, comm=None):
         from ..pane.base import RerenderError
 
-        if self.ncols:
-            width = int(float(self.width)/self.ncols)
+        if self.ncols and self.width:
+            width = self.width/self.ncols
         else:
             width = 0
 
-        if self.nrows:
-            height = int(float(self.height)/self.nrows)
+        if self.nrows and self.height:
+            height = self.height/self.nrows
         else:
             height = 0
 
@@ -274,15 +323,20 @@ class GridSpec(Panel):
             y1 = (self.nrows) if y1 is None else y1
             r, c, h, w = (y0, x0, y1-y0, x1-x0)
 
+            properties = {}
             if self.sizing_mode in ['fixed', None]:
-                properties = {'width': w*width, 'height': h*height}
+                if width:
+                    properties['width'] = int(w*width)
+                if height:
+                    properties['height'] = int(h*height)
             else:
-                properties = {'sizing_mode': self.sizing_mode}
-                if 'width' in self.sizing_mode:
-                    properties['height'] = h*height
-                elif 'height' in self.sizing_mode:
-                    properties['width'] = w*width
-            obj.param.set_param(**{k: v for k, v in properties.items()
+                properties['sizing_mode'] = self.sizing_mode
+                if 'width' in self.sizing_mode and height:
+                    properties['height'] = int(h*height)
+                elif 'height' in self.sizing_mode and width:
+                    properties['width'] = int(w*width)
+
+            obj.param.update(**{k: v for k, v in properties.items()
                                    if not obj.param[k].readonly})
 
             if obj in old_objects:
@@ -293,13 +347,17 @@ class GridSpec(Panel):
                 except RerenderError:
                     return self._get_objects(model, current_objects[:i], doc, root, comm)
 
-            if isinstance(child, BkBox) and len(child.children) == 1:
+            if isinstance(child, BkFlexBox) and len(child.children) == 1:
                 child.children[0].update(**properties)
             else:
                 child.update(**properties)
             children.append((child, r, c, h, w))
         return children
 
+    def _compute_sizing_mode(self, children, props):
+        children = [child for (child, _, _, _, _) in children]
+        return super()._compute_sizing_mode(children, props)
+
     @property
     def _xoffset(self):
         min_xidx = [x0 for (_, x0, _, _) in self.objects if x0 is not None]
@@ -323,7 +381,7 @@ class GridSpec(Panel):
                     grid[y, x] = {((y0, x0, y1, x1), obj)}
         return grid
 
-    def _cleanup(self, root):
+    def _cleanup(self, root: Model | None = None) -> None:
         super()._cleanup(root)
         for p in self.objects.values():
             p._cleanup(root)
@@ -385,9 +443,8 @@ class GridSpec(Panel):
 
         subgrid = self._object_grid[yidx, xidx]
         if isinstance(subgrid, np.ndarray):
-            params = self.param.values()
-            params['objects'] = OrderedDict([list(o)[0] for o in subgrid.flatten()])
-            gspec = type(self)(**params)
+            objects = OrderedDict([list(o)[0] for o in subgrid.flatten()])
+            gspec = self.clone(objects=objects)
             xoff, yoff = gspec._xoffset, gspec._yoffset
             adjusted = []
             for (y0, x0, y1, x1), obj in gspec.objects.items():
