@@ -10,9 +10,9 @@ from typing import (
 
 import param
 
-from bokeh.models import Panel as BkPanel, Spacer as BkSpacer
+from bokeh.models import Spacer as BkSpacer, TabPanel as BkTabPanel
 
-from ..models import Tabs as BkTabs
+from ..models.tabs import Tabs as BkTabs
 from ..viewable import Layoutable
 from .base import NamedListPanel
 
@@ -55,18 +55,18 @@ class Tabs(NamedListPanel):
 
     _bokeh_model: ClassVar[Type[Model]] = BkTabs
 
+    _direction: ClassVar[str | None] = 'vertical'
+
     _js_transforms: ClassVar[Mapping[str, str]] = {'tabs': """
     var ids = [];
     for (var t of value) {{ ids.push(t.id) }};
     var value = ids;
     """}
 
-    _linked_props: ClassVar[List[str]] = ['active', 'tabs']
-
     _manual_params: ClassVar[List[str]] = ['closable']
 
     _rename: ClassVar[Mapping[str, str | None]] = {
-        'name': None, 'objects': 'tabs', 'dynamic': None
+        'closable': None, 'dynamic': None, 'name': None, 'objects': 'tabs'
     }
 
     _source_transforms: ClassVar[Mapping[str, str | None]] = {
@@ -139,6 +139,10 @@ class Tabs(NamedListPanel):
                 self.param.trigger('objects')
                 return
 
+    def _compute_sizing_mode(self, children, props):
+        children = [child.child for child in children]
+        return super()._compute_sizing_mode(children, props)
+
     #----------------------------------------------------------------
     # Model API
     #----------------------------------------------------------------
@@ -197,7 +201,8 @@ class Tabs(NamedListPanel):
                 child = rendered[pref]
             elif hidden:
                 child = BkSpacer(**{k: v for k, v in pane.param.values().items()
-                                    if k in Layoutable.param and v is not None})
+                                    if k in Layoutable.param and v is not None and
+                                    k not in ('name', 'design')})
                 child.tags = ['hidden']
             else:
                 try:
@@ -205,7 +210,7 @@ class Tabs(NamedListPanel):
                 except RerenderError:
                     return self._get_objects(model, current_objects[:i], doc, root, comm)
 
-            panel = panels[pref] = BkPanel(
+            panel = panels[pref] = BkTabPanel(
                 title=name, name=pane.name, child=child, closable=self.closable
             )
             new_models.append(panel)
