@@ -9,6 +9,7 @@ import itertools
 import json
 import os
 import sys
+import textwrap
 import types
 
 from collections import OrderedDict, defaultdict, namedtuple
@@ -16,24 +17,23 @@ from collections.abc import Callable
 from contextlib import contextmanager
 from functools import partial
 from typing import (
-    TYPE_CHECKING, Any, ClassVar, List, Mapping, Optional, Type,
+    TYPE_CHECKING, Any, ClassVar, List, Mapping, Optional, Tuple, Type,
 )
 
 import param
 
-from packaging.version import Version
-from param.parameterized import classlist, discard_events
+from param.parameterized import classlist, discard_events, iscoroutinefunction
 
 from .config import config
-from .io import init_doc, state
+from .io import state
 from .layout import (
     Column, Panel, Row, Spacer, Tabs,
 )
 from .pane.base import PaneBase, ReplacementPane
 from .reactive import Reactive
 from .util import (
-    abbreviated_repr, bokeh_version, classproperty, full_groupby, fullpath,
-    get_method_owner, is_parameterized, param_name, recursive_parameterized,
+    abbreviated_repr, eval_function, full_groupby, fullpath, get_method_owner,
+    is_parameterized, param_name, recursive_parameterized,
 )
 from .viewable import Layoutable, Viewable
 from .widgets import (
@@ -164,7 +164,7 @@ class Param(PaneBase):
         for example lambda x: x[1].label[::-1] will sort by the reversed
         label.""")
 
-    width = param.Integer(default=300, allow_None=True, bounds=(0, None), doc="""
+    width = param.Integer(default=None, allow_None=True, bounds=(0, None), doc="""
         Width of widgetbox the parameter widgets are displayed in.""")
 
     widgets = param.Dict(doc="""
@@ -178,7 +178,7 @@ class Param(PaneBase):
         param.CalendarDate:      DatePicker,
         param.Color:             ColorPicker,
         param.Date:              DatetimeInput,
-        param.DateRange:         DateRangeSlider,
+        param.DateRange:         DatetimeRangeSlider,
         param.CalendarDateRange: DateRangeSlider,
         param.DataFrame:         DataFrameWidget,
         param.Dict:              LiteralInputTyped,
@@ -200,15 +200,16 @@ class Param(PaneBase):
     if hasattr(param, 'Event'):
         mapping[param.Event] = Button
 
-    if bokeh_version >= Version('2.4.3'):
-        mapping[param.DateRange] = DatetimeRangeSlider
-
     priority: ClassVar[float | bool | None] = 0.1
 
-    _unpack: ClassVar[bool] = True
+    _ignored_refs: ClassVar[Tuple[str]] = ('object',)
+
+    _linkable_properties: ClassVar[Tuple[str]] = ()
 
     _rerender_params: ClassVar[List[str]] = []
 
+    _unpack: ClassVar[bool] = True
+
     def __init__(self, object=None, **params):
         if isinstance(object, param.Parameter):
             if 'show_name' not in params:
@@ -252,14 +253,6 @@ class Param(PaneBase):
             'hide_constant'])
         self._update_widgets()
 
-    @classproperty
-    def _mapping(cls):
-        cls.param.warning(
-            "Param._mapping is now deprecated in favor of the public "
-            "Param.mapping attribute. Update your code accordingly."
-        )
-        return cls.mapping
-
     def __repr__(self, depth=0):
         cls = type(self).__name__
         obj_cls = type(self.object).__name__
@@ -267,7 +260,7 @@ class Param(PaneBase):
         parameters = [k for k in params if k != 'name']
         params = []
         for p, v in sorted(self.param.values().items()):
-            if v is self.param[p].default: continue
+            if v == self.param[p].default: continue
             elif v is None: continue
             elif isinstance(v, str) and v == '': continue
             elif p == 'object' or (p == 'name' and (v.startswith(obj_cls) or v.startswith(cls))): continue
@@ -286,7 +279,7 @@ class Param(PaneBase):
 
     @property
     def _synced_params(self):
-        ignored_params = ['default_layout', 'loading']
+        ignored_params = ['default_layout', 'loading', 'background']
         return [p for p in Layoutable.param if p not in ignored_params]
 
     def _update_widgets(self, *events):
@@ -319,10 +312,10 @@ class Param(PaneBase):
             self.parameters = parameters
             return
 
-        for cb in list(self._callbacks):
+        for cb in list(self._internal_callbacks):
             if cb.inst in self._widget_box.objects:
                 cb.inst.param.unwatch(cb)
-                self._callbacks.remove(cb)
+                self._internal_callbacks.remove(cb)
 
         # Construct widgets
         if self.object is None:
@@ -394,7 +387,7 @@ class Param(PaneBase):
             watchers = [selector.param.watch(update_pane, 'value')]
             if toggle:
                 watchers.append(toggle.param.watch(toggle_pane, 'value'))
-            self._callbacks += watchers
+            self._internal_callbacks += watchers
 
             if self.expand:
                 if self.expand_button:
@@ -471,6 +464,9 @@ class Param(PaneBase):
             if hasattr(widget_class, 'fixed_end') and getattr(p_obj, 'bounds', None):
                 kw['fixed_end'] = p_obj.bounds[1]
 
+        if p_obj.doc:
+            kw['description'] = textwrap.dedent(p_obj.doc.lstrip())
+
         # Update kwargs
         kw.update(kw_widget)
 
@@ -486,7 +482,7 @@ class Param(PaneBase):
         widget._param_pane = self
         widget._param_name = p_name
 
-        watchers = self._callbacks
+        watchers = self._internal_callbacks
 
         def link_widget(change):
             if p_name in self._updating:
@@ -566,8 +562,8 @@ class Param(PaneBase):
                 def action(event):
                     change.new(self.object)
                 watchers[0] = widget.param.watch(action, 'clicks')
-                idx = self._callbacks.index(prev_watcher)
-                self._callbacks[idx] = watchers[0]
+                idx = self._internal_callbacks.index(prev_watcher)
+                self._internal_callbacks[idx] = watchers[0]
                 return
             elif kw_widget.get('throttled', False) and hasattr(widget, 'value_throttled'):
                 updates['value_throttled'] = change.new
@@ -606,13 +602,19 @@ class Param(PaneBase):
             options = options.values()
         if ((is_parameterized(value) or any(is_parameterized(o) for o in options))
             and (self.expand_button or (self.expand_button is None and not self.expand))):
-            widget.margin = (5, 0, 5, 10)
-            toggle = Toggle(name='\u22EE', button_type='primary',
-                            disabled=not is_parameterized(value), max_height=30,
-                            max_width=20, height_policy='fit', align='end',
-                            margin=(0, 0, 5, 10))
-            widget.width = self._widget_box.width-60
-            return Row(widget, toggle, width_policy='max', margin=0)
+            toggle = Toggle(
+                name='\u22EE', button_type='primary',
+                disabled=not is_parameterized(value), max_height=30,
+                max_width=20, height_policy='fit', align='end',
+                margin=(0, 0, 5, 10)
+            )
+            width = widget.width
+            widget.param.update(
+                margin=(5, 0, 5, 10),
+                sizing_mode='stretch_width',
+                width=None
+            )
+            return Row(widget, toggle, width=width, margin=0)
         else:
             return widget
 
@@ -656,7 +658,7 @@ class Param(PaneBase):
 
     def _rerender_widget(self, p_name):
         watchers = []
-        for w in self._callbacks:
+        for w in self._internal_callbacks:
             if w.inst is self._widgets[p_name]:
                 w.inst.param.unwatch(w)
             else:
@@ -709,6 +711,15 @@ class Param(PaneBase):
                 return wtype(pobj)
             return wtype
 
+    def get_root(
+        self, doc: Optional[Document] = None, comm: Comm | None = None,
+        preprocess: bool = True
+    ) -> Model:
+        root = super().get_root(doc, comm, preprocess)
+        ref = root.ref['id']
+        self._models[ref] = (root, None)
+        return root
+
     def select(self, selector=None):
         """
         Iterates over the Viewable and any potential children in the
@@ -726,33 +737,6 @@ class Param(PaneBase):
         """
         return super().select(selector) + self.layout.select(selector)
 
-    def get_root(
-        self, doc: Optional[Document] = None, comm: Optional[Comm] = None,
-        preprocess: bool = True
-    ) -> Model:
-        """
-        Returns the root model and applies pre-processing hooks
-
-        Arguments
-        ---------
-        doc: bokeh.Document
-          Bokeh document the bokeh model will be attached to.
-        comm: pyviz_comms.Comm
-          Optional pyviz_comms when working in notebook
-        preprocess: boolean (default=True)
-          Whether to run preprocessing hooks
-
-        Returns
-        -------
-        Returns the bokeh model corresponding to this panel object
-        """
-        doc = init_doc(doc)
-        root = self.layout.get_root(doc, comm, preprocess)
-        ref = root.ref['id']
-        self._models[ref] = (root, None)
-        state._views[ref] = (self, root, doc, comm)
-        return root
-
 
 class ParamMethod(ReplacementPane):
     """
@@ -808,14 +792,7 @@ class ParamMethod(ReplacementPane):
 
     @classmethod
     def eval(self, function):
-        args, kwargs = (), {}
-        if hasattr(function, '_dinfo'):
-            arg_deps = function._dinfo['dependencies']
-            kw_deps = function._dinfo.get('kw', {})
-            if kw_deps or any(isinstance(d, param.Parameter) for d in arg_deps):
-                args = (getattr(dep.owner, dep.name) for dep in arg_deps)
-                kwargs = {n: getattr(dep.owner, dep.name) for n, dep in kw_deps.items()}
-        return function(*args, **kwargs)
+        return eval_function(function)
 
     async def _eval_async(self, awaitable):
         try:
@@ -845,13 +822,13 @@ class ParamMethod(ReplacementPane):
 
     def _update_pane(self, *events):
         callbacks = []
-        for watcher in self._callbacks:
+        for watcher in self._internal_callbacks:
             obj = watcher.inst if watcher.inst is None else watcher.cls
             if obj is self:
                 callbacks.append(watcher)
                 continue
             obj.param.unwatch(watcher)
-        self._callbacks = callbacks
+        self._internal_callbacks = callbacks
         self._link_object_params()
         self._replace_pane()
 
@@ -866,7 +843,7 @@ class ParamMethod(ReplacementPane):
                 new_deps = parameterized.param.method_dependencies(self.object.__name__)
                 for p in list(deps):
                     if p in new_deps: continue
-                    watchers = self._callbacks
+                    watchers = self._internal_callbacks
                     for w in list(watchers):
                         if (w.inst is p.inst and w.cls is p.cls and
                             p.name in w.parameter_names):
@@ -881,7 +858,7 @@ class ParamMethod(ReplacementPane):
                     pobj = p.cls if p.inst is None else p.inst
                     ps = [_p.name for _p in params]
                     watcher = pobj.param.watch(update_pane, ps, p.what)
-                    self._callbacks.append(watcher)
+                    self._internal_callbacks.append(watcher)
                     for p in params:
                         deps.append(p)
             self._replace_pane()
@@ -895,7 +872,7 @@ class ParamMethod(ReplacementPane):
                 if props:
                     pobj.jslink(self._inner_layout, **props)
             watcher = pobj.param.watch(update_pane, ps, p.what)
-            self._callbacks.append(watcher)
+            self._internal_callbacks.append(watcher)
 
     def _get_model(
         self, doc: Document, root: Optional[Model] = None,
@@ -940,7 +917,7 @@ class ParamFunction(ParamMethod):
     def _link_object_params(self):
         deps = getattr(self.object, '_dinfo', {})
         dep_params = list(deps.get('dependencies', [])) + list(deps.get('kw', {}).values())
-        if not dep_params and not self.lazy and not self.defer_load:
+        if not dep_params and not self.lazy and not self.defer_load and not iscoroutinefunction(self.object):
             fn = getattr(self.object, '__bound_function__', self.object)
             fn_name = getattr(fn, '__name__', repr(self.object))
             self.param.warning(
@@ -964,7 +941,7 @@ class ParamFunction(ParamMethod):
                          if dep.name in pobj._linkable_params}
                 if props:
                     pobj.jslink(self._inner_layout, **props)
-            self._callbacks.append(watcher)
+            self._internal_callbacks.append(watcher)
 
     #----------------------------------------------------------------
     # Public API
@@ -975,7 +952,11 @@ class ParamFunction(ParamMethod):
         if isinstance(obj, types.FunctionType):
             if hasattr(obj, '_dinfo'):
                 return True
-            if kwargs.get('defer_load') or (cls.param.defer_load.default is None and config.defer_load):
+            if (
+                kwargs.get('defer_load') or
+                (cls.param.defer_load.default is None and config.defer_load) or
+                iscoroutinefunction(obj)
+            ):
                 return True
             return None
         return False
@@ -1008,7 +989,7 @@ class JSONInit(param.Parameterized):
         Optional path to a JSON file containing the parameter settings.""")
 
     def __call__(self, parameterized):
-        warnobj = param.main if isinstance(parameterized, type) else parameterized
+        warnobj = param.main.param if isinstance(parameterized, type) else parameterized.param
         param_class = (parameterized if isinstance(parameterized, type)
                        else parameterized.__class__)
 
@@ -1054,7 +1035,7 @@ def link_param_method(root_view, root_model):
 
     for widget in widgets:
         for method in methods:
-            for cb in method._callbacks:
+            for cb in method._internal_callbacks:
                 pobj = cb.cls if cb.inst is None else cb.inst
                 if widget._param_pane.object is pobj and widget._param_name in cb.parameter_names:
                     if isinstance(widget, DiscreteSlider):
