@@ -13,7 +13,7 @@ import sys
 import textwrap
 
 from collections import Counter, defaultdict, namedtuple
-from functools import partial
+from functools import lru_cache, partial
 from pprint import pformat
 from typing import (
     TYPE_CHECKING, Any, Callable, ClassVar, Dict, List, Mapping, Optional, Set,
@@ -26,17 +26,25 @@ import param
 
 from bokeh.core.property.descriptors import UnsetValueError
 from bokeh.model import DataModel
+from bokeh.models import ImportedStyleSheet
 from packaging.version import Version
 from param.parameterized import ParameterizedMetaclass, Watcher
 
 from .io.document import unlocked
 from .io.model import hold
 from .io.notebook import push
+from .io.resources import (
+    CDN_DIST, loading_css, patch_stylesheet, process_raw_css,
+    resolve_stylesheet,
+)
 from .io.state import set_curdoc, state
 from .models.reactive_html import (
     DOMEvent, ReactiveHTML as _BkReactiveHTML, ReactiveHTMLParser,
 )
-from .util import edit_readonly, escape, updating
+from .util import (
+    BOKEH_JS_NAT, classproperty, edit_readonly, escape, eval_function,
+    extract_dependencies, updating,
+)
 from .viewable import Layoutable, Renderable, Viewable
 
 if TYPE_CHECKING:
@@ -96,9 +104,14 @@ class Syncable(Renderable):
     _source_transforms: ClassVar[Mapping[str, str | None]] = {}
     _target_transforms: ClassVar[Mapping[str, str | None]] = {}
 
+    # A list of stylesheets specified as paths relative to the
+    # panel/dist directory
+    _stylesheets: ClassVar[List[str]] = []
+
     __abstract = True
 
     def __init__(self, **params):
+        self._themer = None
         super().__init__(**params)
 
         # Useful when updating model properties which trigger potentially
@@ -117,12 +130,33 @@ class Syncable(Renderable):
 
         # Sets up watchers to process manual updates to models
         if self._manual_params:
-            self.param.watch(self._update_manual, self._manual_params)
+            self._internal_callbacks.append(
+                self.param.watch(self._update_manual, self._manual_params)
+            )
 
     #----------------------------------------------------------------
     # Model API
     #----------------------------------------------------------------
 
+    @classproperty
+    @lru_cache(maxsize=None)
+    def _property_mapping(cls):
+        rename = {}
+        for scls in cls.__mro__[::-1]:
+            if issubclass(scls, Syncable):
+                rename.update(scls._rename)
+        return rename
+
+    @property
+    def _linked_properties(self) -> Tuple[str]:
+        return tuple(
+            self._property_mapping.get(p, p) for p in self.param
+            if p not in Viewable.param and self._property_mapping.get(p, p) is not None
+        )
+
+    def _get_properties(self, doc: Document) -> Dict[str, Any]:
+        return self._process_param_change(self._init_params())
+
     def _process_property_change(self, msg: Dict[str, Any]) -> Dict[str, Any]:
         """
         Transform bokeh model property changes into parameter updates.
@@ -131,7 +165,7 @@ class Syncable(Renderable):
         _rename class level attribute to map between parameter and
         property names.
         """
-        inverted = {v: k for k, v in self._rename.items()}
+        inverted = {v: k for k, v in self._property_mapping.items()}
         return {inverted.get(k, k): v for k, v in msg.items()}
 
     def _process_param_change(self, msg: Dict[str, Any]) -> Dict[str, Any]:
@@ -142,22 +176,43 @@ class Syncable(Renderable):
         _rename class level attribute to map between parameter and
         property names.
         """
-        properties = {self._rename.get(k) or k: v for k, v in msg.items()
-                      if self._rename.get(k, False) is not None}
+        properties = {
+            self._property_mapping.get(k) or k: v for k, v in msg.items()
+            if self._property_mapping.get(k, False) is not None and
+            k not in self._manual_params
+        }
         if 'width' in properties and self.sizing_mode is None:
             properties['min_width'] = properties['width']
         if 'height' in properties and self.sizing_mode is None:
             properties['min_height'] = properties['height']
+        if 'stylesheets' in properties:
+            from .config import config
+            stylesheets = [loading_css(), f'{CDN_DIST}css/loading.css']
+            stylesheets += process_raw_css(config.raw_css)
+            stylesheets += config.css_files
+            stylesheets += [
+                resolve_stylesheet(self, css_file, '_stylesheets')
+                for css_file in self._stylesheets
+            ]
+            stylesheets += properties['stylesheets']
+            wrapped = []
+            for stylesheet in stylesheets:
+                if isinstance(stylesheet, str) and stylesheet.endswith('.css'):
+                    stylesheet = ImportedStyleSheet(url=stylesheet)
+                wrapped.append(stylesheet)
+            properties['stylesheets'] = wrapped
         return properties
 
     @property
     def _linkable_params(self) -> List[str]:
         """
-        Parameters that can be linked in JavaScript via source
-        transforms.
+        Parameters that can be linked in JavaScript via source transforms.
         """
-        return [p for p in self._synced_params if self._rename.get(p, False) is not None
-                and self._source_transforms.get(p, False) is not None] + ['loading']
+        return [
+            p for p in self._synced_params if self._rename.get(p, False) is not None
+            and self._source_transforms.get(p, False) is not None and
+            p not in ('design', 'stylesheets')
+        ]
 
     @property
     def _synced_params(self) -> List[str]:
@@ -165,18 +220,20 @@ class Syncable(Renderable):
         Parameters which are synced with properties using transforms
         applied in the _process_param_change method.
         """
-        ignored = ['default_layout', 'loading']
+        ignored = ['default_layout', 'loading', 'background']
         return [p for p in self.param if p not in self._manual_params+ignored]
 
     def _init_params(self) -> Dict[str, Any]:
-        return {k: v for k, v in self.param.values().items()
-                if k in self._synced_params and v is not None}
+        return {
+            k: v for k, v in self.param.values().items()
+            if k in self._synced_params and v is not None
+        }
 
     def _link_params(self) -> None:
         params = self._synced_params
         if params:
             watcher = self.param.watch(self._param_change, params)
-            self._callbacks.append(watcher)
+            self._internal_callbacks.append(watcher)
 
     def _link_props(
         self, model: Model, properties: List[str] | List[Tuple[str, str]],
@@ -259,6 +316,11 @@ class Syncable(Renderable):
                 continue
             if not model.lookup(attr).property.matches(model_val, value):
                 attrs.append(attr)
+
+            # Do not apply model change that is in flight
+            if attr in self._events:
+                del self._events[attr]
+
         try:
             model.update(**msg)
         finally:
@@ -292,20 +354,19 @@ class Syncable(Renderable):
             except Exception:
                 pass
 
+    def _update_properties(
+        self, *events: param.parameterized.Event, doc: Document
+    ) -> Dict[str, Any]:
+        changes = {event.name: event.new for event in events}
+        return self._process_param_change(changes)
+
     def _param_change(self, *events: param.parameterized.Event) -> None:
-        msgs = []
-        for event in events:
-            msg = self._process_param_change({event.name: event.new})
-            if msg:
-                msgs.append(msg)
-
         named_events = {event.name: event for event in events}
-        msg = {k: v for msg in msgs for k, v in msg.items()}
-        if not msg:
-            return
-
         for ref, (model, _) in self._models.copy().items():
-            self._apply_update(named_events, msg, model, ref)
+            properties = self._update_properties(*events, doc=model.document)
+            if not properties:
+                return
+            self._apply_update(named_events, properties, model, ref)
 
     def _process_events(self, events: Dict[str, Any]) -> None:
         self._log('received events %s', events)
@@ -463,6 +524,158 @@ class Reactive(Syncable, Viewable):
     the parameters to other objects.
     """
 
+    # Parameter values which should not be treated like references
+    _ignored_refs: ClassVar[List[str]] = []
+
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'design': None, 'loading': None
+    }
+
+    __abstract = True
+
+    def __init__(self, **params):
+        params, refs = self._extract_refs(params)
+        super().__init__(**params)
+        self._refs = refs
+        self._setup_refs(refs)
+
+    def _resolve_ref(self, pname, value):
+        from .depends import param_value_if_widget
+        ref = None
+        value = param_value_if_widget(value)
+        if isinstance(value, param.Parameter):
+            ref = value
+            value = getattr(value.owner, value.name)
+        elif hasattr(value, '_dinfo'):
+            ref = value
+            value = eval_function(value)
+        return ref, value
+
+    def _validate_ref(self, pname, value):
+        pobj = self.param[pname]
+        pobj._validate(value)
+        if isinstance(pobj, param.Dynamic) and callable(value) and hasattr(value, '_dinfo'):
+            raise ValueError(
+                'Dynamic parameters should not capture functions with dependencies.'
+            )
+
+    def _extract_refs(self, params):
+        processed, refs = {}, {}
+        for pname, value in params.items():
+            if pname not in self.param or pname in self._ignored_refs:
+                processed[pname] = value
+                continue
+
+            # Only consider extracting reference if the provided value is not
+            # a valid value for the parameter (or no validation was defined)
+            try:
+                self._validate_ref(pname, value)
+            except Exception:
+                pass
+            else:
+                pobj = self.param[pname]
+                if type(pobj) is not param.Parameter:
+                    processed[pname] = value
+                    continue
+
+            # Resolve references, allowing for Widget, Parameter and
+            # objects with dependencies
+            ref, value = self._resolve_ref(pname, value)
+            if ref is not None:
+                refs[pname] = ref
+            processed[pname] = value
+        return processed, refs
+
+    def _sync_refs(self, *events):
+        from .config import config
+        if config.loading_indicator:
+            self.loading = True
+        updates = {}
+        for pname, p in self._refs.items():
+            if isinstance(p, param.Parameter):
+                deps = (p,)
+            else:
+                deps = extract_dependencies(p)
+            # Skip updating value if dependency has not changed
+            if not any((dep.owner is e.obj and dep.name == e.name) for dep in deps for e in events):
+                continue
+            if isinstance(p, param.Parameter):
+                updates[pname] = getattr(p.owner, p.name)
+            else:
+                updates[pname] = eval_function(p)
+        if config.loading_indicator:
+            updates['loading'] = False
+        with param.edit_constant(self):
+            self.param.update(updates)
+
+    def _setup_refs(self, refs):
+        groups = defaultdict(list)
+        for pname, p in refs.items():
+            if isinstance(p, param.Parameter):
+                groups[p.owner].append(p.name)
+            else:
+                for sp in extract_dependencies(p):
+                    groups[sp.owner].append(sp.name)
+        for owner, pnames in groups.items():
+            self._internal_callbacks.append(
+                owner.param.watch(self._sync_refs, list(set(pnames)))
+            )
+
+    #----------------------------------------------------------------
+    # Private API
+    #----------------------------------------------------------------
+
+    def _get_properties(self, doc: Document) -> Dict[str, Any]:
+        params, _ = self._design.params(self, doc) if self._design else ({}, None)
+        for k, v in self._init_params().items():
+            if k in ('stylesheets', 'tags') and k in params:
+                params[k] = v = params[k] + v
+            elif k not in params or self.param[k].default is not v:
+                params[k] = v
+        properties = self._process_param_change(params)
+        if 'stylesheets' not in properties:
+            return properties
+        if doc:
+            state._stylesheets[doc] = cache = state._stylesheets.get(doc, {})
+        else:
+            cache = {}
+        if doc and 'dist_url' in doc._template_variables:
+            dist_url = doc._template_variables['dist_url']
+        else:
+            dist_url = CDN_DIST
+        stylesheets = []
+        for stylesheet in properties['stylesheets']:
+            if isinstance(stylesheet, ImportedStyleSheet):
+                if stylesheet.url in cache:
+                    stylesheet = cache[stylesheet.url]
+                else:
+                    cache[stylesheet.url] = stylesheet
+                patch_stylesheet(stylesheet, dist_url)
+            stylesheets.append(stylesheet)
+        properties['stylesheets'] = stylesheets
+        return properties
+
+    def _update_properties(self, *events: param.parameterized.Event, doc: Document) -> Dict[str, Any]:
+        params, _ = self._design.params(self, doc) if self._design else ({}, None)
+        changes = {event.name: event.new for event in events}
+        if 'stylesheets' in changes and 'stylsheets' in params:
+            changes['stylesheets'] = params['stylesheets'] + changes['stylesheets']
+        return self._process_param_change(changes)
+
+    def _update_model(
+        self, events: Dict[str, param.parameterized.Event], msg: Dict[str, Any],
+        root: Model, model: Model, doc: Document, comm: Optional[Comm]
+    ) -> None:
+        if 'stylesheets' in msg:
+            if doc and 'dist_url' in doc._template_variables:
+                dist_url = doc._template_variables['dist_url']
+            else:
+                dist_url = CDN_DIST
+            for stylesheet in msg['stylesheets']:
+                if isinstance(stylesheet, ImportedStyleSheet):
+                    patch_stylesheet(stylesheet, dist_url)
+        super()._update_model(events, msg, root, model, doc, comm)
+
     #----------------------------------------------------------------
     # Public API
     #----------------------------------------------------------------
@@ -580,7 +793,7 @@ class Reactive(Syncable, Viewable):
         controls = Param(self.param, parameters=params, default_layout=WidgetBox,
                          name='Controls', **kwargs)
         layout_params = [p for p in linkable if p in Viewable.param]
-        if 'name' not in layout_params and self._rename.get('name', False) is not None and not parameters:
+        if 'name' not in layout_params and self._property_mapping.get('name', False) is not None and not parameters:
             layout_params.insert(0, 'name')
         style = Param(self.param, parameters=layout_params, default_layout=WidgetBox,
                       name='Layout', **kwargs)
@@ -702,10 +915,13 @@ class SyncableData(Reactive):
         super().__init__(**params)
         self._data = None
         self._processed = None
-        self.param.watch(self._validate, self._data_params)
+        callbacks = [self.param.watch(self._validate, self._data_params)]
         if self._data_params:
-            self.param.watch(self._update_cds, self._data_params)
-        self.param.watch(self._update_selected, 'selection')
+            callbacks.append(
+                self.param.watch(self._update_cds, self._data_params)
+            )
+        callbacks.append(self.param.watch(self._update_selected, 'selection'))
+        self._internal_callbacks += callbacks
         self._validate()
         self._update_cds()
 
@@ -1042,6 +1258,8 @@ class ReactiveData(SyncableData):
     parameter between frontend and backend using a ColumnDataSource.
     """
 
+    __abstract = True
+
     def __init__(self, **params):
         super().__init__(**params)
 
@@ -1055,7 +1273,8 @@ class ReactiveData(SyncableData):
         converted: List | np.ndarray | None = None
         if dtype.kind == 'M':
             if values.dtype.kind in 'if':
-                converted = (values * 10e5).astype(dtype)
+                NATs = values == BOKEH_JS_NAT
+                converted = np.where(NATs, np.nan, values * 10e5).astype(dtype)
         elif dtype.kind == 'O':
             if (all(isinstance(ov, dt.date) for ov in old_values) and
                 not all(isinstance(iv, dt.date) for iv in values)):
@@ -1100,10 +1319,18 @@ class ReactiveData(SyncableData):
             values = self._convert_column(
                 np.asarray(values), old_raw[col]
             )
-            try:
-                isequal = (old_raw[col] == values).all() # type: ignore
-            except Exception:
-                isequal = False
+
+            isequal = None
+            if hasattr(old_raw, 'columns') and isinstance(values, np.ndarray):
+                try:
+                    isequal = np.array_equal(old_raw[col], values, equal_nan=True)
+                except Exception:
+                    pass
+            if isequal is None:
+                try:
+                    isequal = (old_raw[col] == values).all() # type: ignore
+                except Exception:
+                    isequal = False
             if not isequal:
                 self._update_column(col, values)
                 updated = True
@@ -1430,7 +1657,8 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         """
         return (
             cls._extension_name is None or
-            cls._extension_name in ReactiveHTMLMetaclass._loaded_extensions
+            (cls._extension_name in ReactiveHTMLMetaclass._loaded_extensions and
+             (state._extensions is None or (cls._extension_name in state._extensions)))
         )
 
     def _cleanup(self, root: Model | None = None) -> None:
@@ -1441,7 +1669,9 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
 
     @property
     def _linkable_params(self) -> List[str]:
-        return [p for p in super()._linkable_params if p not in self._parser.children.values()]
+        return [
+            p for p in super()._linkable_params if p not in self._parser.children.values() and
+            p not in ('loading')]
 
     @property
     def _child_names(self):
@@ -1453,6 +1683,15 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
     ) -> Dict[str, List[Model]]:
         return children
 
+    def _process_param_change(self, params):
+        props = super()._process_param_change(params)
+        if 'stylesheets' in params:
+            css = getattr(self, '__css__', []) or []
+            props['stylesheets'] = [
+                ImportedStyleSheet(url=ss) for ss in css
+            ] + props['stylesheets']
+        return props
+
     def _init_params(self) -> Dict[str, Any]:
         ignored = list(Reactive.param)
         for child in self._parser.children.values():
@@ -1642,6 +1881,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                 html = html.replace('${%s}' % child_name, '')
         return html, parser.nodes, p_attrs
 
+    @property
     def _linked_properties(self) -> List[str]:
         linked_properties = [p for pss in self._attrs.values() for _, ps, _ in pss for p in ps]
         for scripts in self._scripts.values():
@@ -1654,14 +1894,13 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         for children_param in self._parser.children.values():
             if children_param in self._data_model.properties():
                 linked_properties.append(children_param)
-        return linked_properties
+        return tuple(linked_properties)
 
     def _get_model(
         self, doc: Document, root: Optional[Model] = None,
         parent: Optional[Model] = None, comm: Optional[Comm] = None
     ) -> Model:
-        properties = self._process_param_change(self._init_params())
-        model = _BkReactiveHTML(**properties)
+        model = _BkReactiveHTML(**self._get_properties(doc))
         if comm and not self._loaded():
             self.param.warning(
                 f'{type(self).__name__} was not imported on instantiation and may not '
@@ -1688,7 +1927,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                 v.tags.append(f"__ref:{root.ref['id']}")
         model.update(children=self._get_children(doc, root, model, comm))
         self._register_events('dom_event', model=model, doc=doc, comm=comm)
-        self._link_props(data_model, self._linked_properties(), doc, root, comm)
+        self._link_props(data_model, self._linked_properties, doc, root, comm)
         self._models[root.ref['id']] = (model, parent)
         return model
 
