@@ -13,6 +13,7 @@ from bokeh.document import Document
 from bokeh.io.doc import patch_curdoc
 from bokeh.models import Div
 
+from panel.depends import bind, depends
 from panel.layout import Tabs, WidgetBox
 from panel.reactive import Reactive, ReactiveHTML
 from panel.viewable import Viewable
@@ -123,7 +124,7 @@ def test_text_input_controls():
     assert len(controls) == 2
     wb1, wb2 = controls
     assert isinstance(wb1, WidgetBox)
-    assert len(wb1) == 6
+    assert len(wb1) == 7
     name, disabled, *(ws) = wb1
 
     assert isinstance(name, StaticText)
@@ -143,14 +144,96 @@ def test_text_input_controls():
             assert w.value == "Test placeholder..."
         elif w.name == 'Max length':
             assert isinstance(w, IntInput)
+        elif w.name == 'Description':
+            assert isinstance(w, TextInput)
+            text_input.description = "Test description..."
+            assert w.value == "Test description..."
         else:
             not_checked.append(w)
 
     assert not not_checked
 
     assert isinstance(wb2, WidgetBox)
-    assert len(wb2) == len(list(Viewable.param)) + 1
 
+    params1 = {w.name.replace(" ", "_").lower() for w in wb2 if len(w.name)}
+    params2 = set(Viewable.param) - {"background", "design", "stylesheets", "loading"}
+    # Background should be moved when Layoutable.background is removed.
+
+    assert not len(params1 - params2)
+    assert not len(params2 - params1)
+
+
+def test_pass_widget_by_reference():
+    int_input = IntInput(start=0, end=400, value=42)
+    text_input = TextInput(width=int_input)
+
+    assert text_input.width == 42
+
+    int_input.value = 101
+
+    assert text_input.width == 101
+
+
+def test_pass_param_by_reference():
+    int_input = IntInput(start=0, end=400, value=42)
+    text_input = TextInput(width=int_input.param.value)
+
+    assert text_input.width == 42
+
+    int_input.value = 101
+
+    assert text_input.width == 101
+
+
+def test_pass_bind_function_by_reference():
+    int_input = IntInput(start=0, end=400, value=42)
+    fn = bind(lambda v: v + 10, int_input)
+    text_input = TextInput(width=fn)
+
+    assert text_input.width == 52
+
+    int_input.value = 101
+
+    assert text_input.width == 111
+
+def test_pass_parameterized_method_by_reference():
+
+    class Test(param.Parameterized):
+
+        a = param.Parameter(default=1)
+        b = param.Parameter(default=2)
+
+        @param.depends('a')
+        def dep_a(self):
+            return self.a
+
+        @param.depends('dep_a', 'b')
+        def dep_ab(self):
+            return self.dep_a() + self.b
+
+    test = Test()
+    int_input = IntInput(start=0, end=400, value=test.dep_ab)
+
+    assert int_input.value == 3
+
+    test.a = 3
+
+    assert int_input.value == 5
+
+    test.b = 5
+
+    assert int_input.value == 8
+
+def test_pass_depends_function_by_reference():
+    int_input = IntInput(start=0, end=400, value=42)
+    fn = depends(int_input)(lambda v: v + 10)
+    text_input = TextInput(width=fn)
+
+    assert text_input.width == 52
+
+    int_input.value = 101
+
+    assert text_input.width == 111
 
 
 def test_text_input_controls_explicit():
@@ -514,7 +597,7 @@ def test_reactive_html_scripts_linked_properties_assignment_operator():
 
                 _scripts = {'render': f'test.onclick = () => {{ data.clicks{sep}{operator}= 1 }}'}
 
-            assert TestScripts()._linked_properties() == ['clicks']
+            assert TestScripts()._linked_properties == ('clicks',)
 
 
 def test_reactive_html_templated_literal_add_loop_id_and_for_loop_var():
