@@ -47,7 +47,7 @@ def test_fast_list_template_updates(page, port):
 
     page.goto(f"http://localhost:{port}", timeout=40_000)
 
-    assert page.text_content(".bk.markdown") == 'Initial'
+    assert page.locator(".markdown").locator("div").text_content() == 'Initial\n'
     md.object = 'Updated'
     time.sleep(0.1)
-    assert page.text_content(".bk.markdown") == 'Updated'
+    assert page.locator(".markdown").locator("div").text_content() == 'Updated\n'
