@@ -2,25 +2,19 @@ import datetime as dt
 import time
 
 import numpy as np
+import pandas as pd
 import pytest
 import requests
 
-from packaging.version import Version
-
-try:
-    import pandas as pd
-
-    from pandas._testing import (
-        makeCustomDataframe, makeMixedDataFrame, makeTimeDataFrame,
-    )
-except ImportError:
-    pytestmark = pytest.mark.skip('pandas not available')
-
 from bokeh.models.widgets.tables import (
     AvgAggregator, CellEditor, CheckboxEditor, DataCube, DateEditor,
     DateFormatter, IntEditor, MinAggregator, NumberEditor, NumberFormatter,
     SelectEditor, StringEditor, StringFormatter, SumAggregator,
 )
+from packaging.version import Version
+from pandas._testing import (
+    makeCustomDataframe, makeMixedDataFrame, makeTimeDataFrame,
+)
 
 from panel.depends import bind
 from panel.io.server import serve
@@ -246,7 +240,7 @@ def test_tabulator_multi_index(document, comm):
     df = makeMixedDataFrame()
     table = Tabulator(df.set_index(['A', 'C']))
 
-    model = table.get_root()
+    model = table.get_root(document, comm)
 
     assert model.configuration['columns'] == [
         {'field': 'A', 'sorter': 'number'},
@@ -263,7 +257,7 @@ def test_tabulator_multi_index_remote_pagination(document, comm):
     df = makeMixedDataFrame()
     table = Tabulator(df.set_index(['A', 'C']), pagination='remote', page_size=3)
 
-    model = table.get_root()
+    model = table.get_root(document, comm)
 
     assert model.configuration['columns'] == [
         {'field': 'A', 'sorter': 'number'},
@@ -276,12 +270,12 @@ def test_tabulator_multi_index_remote_pagination(document, comm):
     assert np.array_equal(model.source.data['C'], np.array(['foo1', 'foo2', 'foo3']))
 
 
-def test_tabulator_expanded_content():
+def test_tabulator_expanded_content(document, comm):
     df = makeMixedDataFrame()
 
     table = Tabulator(df, expanded=[0, 1], row_content=lambda r: r.A)
 
-    model = table.get_root()
+    model = table.get_root(document, comm)
 
     assert len(model.children) == 2
 
@@ -320,12 +314,12 @@ def test_tabulator_index_column(document, comm):
     assert model.columns[0].title == ''
 
 
-def test_tabulator_expanded_content_pagination():
+def test_tabulator_expanded_content_pagination(document, comm):
     df = makeMixedDataFrame()
 
     table = Tabulator(df, expanded=[0, 1], row_content=lambda r: r.A, pagination='remote', page_size=2)
 
-    model = table.get_root()
+    model = table.get_root(document, comm)
 
     assert len(model.children) == 2
 
@@ -334,12 +328,12 @@ def test_tabulator_expanded_content_pagination():
     assert len(model.children) == 0
 
 
-def test_tabulator_expanded_content_embed():
+def test_tabulator_expanded_content_embed(document, comm):
     df = makeMixedDataFrame()
 
     table = Tabulator(df, embed_content=True, row_content=lambda r: r.A)
 
-    model = table.get_root()
+    model = table.get_root(document, comm)
 
     assert len(model.children) == len(df)
 
@@ -752,7 +746,7 @@ def test_tabulator_styling(document, comm):
 
     model = table.get_root(document, comm)
 
-    assert model.styles['data'] == {
+    assert model.cell_styles['data'] == {
         0: {2: [('color', 'black')]},
         1: {2: [('color', 'black')]},
         2: {2: [('color', 'black')]},
@@ -1329,7 +1323,7 @@ def test_tabulator_constant_scalar_filter_client_side(document, comm):
         'C': np.array(['foo3']),
         'D': np.array(['2009-01-05T00:00:00.000000000'],
                       dtype='datetime64[ns]')
-    }, index=np.array([2]))
+    }, index=[2])
     pd.testing.assert_frame_equal(table._processed, expected)
 
 def test_tabulator_constant_scalar_filter_on_index_client_side(document, comm):
@@ -1344,7 +1338,7 @@ def test_tabulator_constant_scalar_filter_on_index_client_side(document, comm):
         'C': np.array(['foo3']),
         'D': np.array(['2009-01-05T00:00:00.000000000'],
                       dtype='datetime64[ns]')
-    }, index=np.array([2]))
+    }, index=[2])
     pd.testing.assert_frame_equal(table._processed, expected)
 
 def test_tabulator_constant_scalar_filter_on_multi_index_client_side(document, comm):
@@ -1378,7 +1372,7 @@ def test_tabulator_constant_list_filter_client_side(document, comm):
         'D': np.array(['2009-01-05T00:00:00.000000000',
                        '2009-01-07T00:00:00.000000000'],
                       dtype='datetime64[ns]')
-    }, index=np.array([2, 4]))
+    }, index=[2, 4])
     pd.testing.assert_frame_equal(table._processed, expected)
 
 def test_tabulator_keywords_filter_client_side(document, comm):
@@ -1394,7 +1388,7 @@ def test_tabulator_keywords_filter_client_side(document, comm):
         'D': np.array(['2009-01-05T00:00:00.000000000',
                        '2009-01-07T00:00:00.000000000'],
                       dtype='datetime64[ns]')
-    }, index=np.array([2, 4]))
+    }, index=[2, 4])
     pd.testing.assert_frame_equal(table._processed, expected)
 
 def test_tabulator_keywords_match_all_filter_client_side(document, comm):
@@ -1592,7 +1586,7 @@ def test_tabulator_widget_scalar_filter(document, comm):
 def test_tabulator_constant_list_filter(document, comm, col):
     df = makeMixedDataFrame()
     # The mixed dataframe has duplicate number values in the B columns,
-    # simplify the test by setting the targetted valued before filtering.
+    # simplify the test by setting the targeted valued before filtering.
     df.at[2, 'B'] = 10.0
     df.at[4, 'B'] = 20.0
     table = Tabulator(df)
@@ -1911,7 +1905,7 @@ def test_tabulator_cell_click_event_error_duplicate_index():
     with pytest.raises(ValueError, match="Found this duplicate index: 'a'"):
         table._process_event(event)
 
-def test_tabulator_styling_empty_dataframe():
+def test_tabulator_styling_empty_dataframe(document, comm):
     df = pd.DataFrame(columns=["A", "B", "C"]).astype({
         "A": float,
         "B": str,
@@ -1922,13 +1916,13 @@ def test_tabulator_styling_empty_dataframe():
         "border-color: #dc3545; border-style: solid" for name, value in x.items()
     ], axis=1)
 
-    model = table.get_root()
+    model = table.get_root(document, comm)
 
     assert model.styles == {}
 
     table.value = pd.DataFrame({'A': [3.14], 'B': ['foo'], 'C': [3]})
 
-    assert model.styles['data'] == {
+    assert model.cell_styles['data'] == {
         0: {
             2: [('border-color', '#dc3545'), ('border-style', 'solid')],
             3: [('border-color', '#dc3545'), ('border-style', 'solid')],
